import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { FormattedMessage } from 'react-intl';
import CopyToClipboard from 'react-copy-to-clipboard';

import Input from 'reactstrap/lib/Input';
import Button from 'reactstrap/lib/Button';

import { IconCopyWhite } from '../../icons/icon-copy/icon-copy-white';
import { IconDotPreloader } from '../../icons/icon-dot-preloader/icon-dot-preloader';

import { DEFAULT_RESET_STATE_TIMEOUT } from '../../../constants';
import './input-with-clipboard.scss';

export class InputWithClipboard extends Component {
  static propTypes = {
    value: PropTypes.string.isRequired,
    isLoading: PropTypes.bool,
    onCopyClick: PropTypes.func,
  };

  static defaultProps = {
    isLoading: false,
    onCopyClick: () => {},
  };

  state = {
    copied: false,
  };

  clipboardTimeout;

  componentWillUnmount() {
    if (this.clipboardTimeout) {
      clearTimeout(this.clipboardTimeout);
    }
  }

  copy = () => {
    this.props.onCopyClick();
    this.setState({ copied: true });
    this.clipboardTimeout = setTimeout(this.resetCopy, DEFAULT_RESET_STATE_TIMEOUT);
  };

  resetCopy = () => {
    this.setState({ copied: false });
  };

  render() {
    const { copied } = this.state;
    const { value, isLoading } = this.props;

    return (
      <div className="d-flex input-with-clipboard">
        <span className="clipboard-text w-100">
          <Input readOnly type="text" value={isLoading ? '' : value} className="clipboard-input px-2 border-0" />
          {isLoading && <IconDotPreloader className="icon-dot h5p text-gray-dark-hint" />}
        </span>

        <CopyToClipboard text={value}>
          <Button color="primary" className="clipboard-button" onClick={this.copy}>
            {copied ? <IconCopyWhite /> : <FormattedMessage id="universal.copy" />}
          </Button>
        </CopyToClipboard>
      </div>
    );
  }
}
