import React, { Component } from 'react';
import PropTypes from 'prop-types';
import Dropdown from 'reactstrap/lib/Dropdown';
import DropdownToggle from 'reactstrap/lib/DropdownToggle';
import DropdownMenu from 'reactstrap/lib/DropdownMenu';
import DropdownItem from 'reactstrap/lib/DropdownItem';
import classNames from 'classnames';

import { LANG_NAMES, LANG_KEYS } from '../../constants';
import { IconArrowDown } from '../icons/icon-arrow-down/icon-arrow-down';
import { IconSelected } from '../icons/icon-selected/icon-selected';
import { changeLocation, getLocationOrigin } from '../../helpers/location';

import './lang-switcher.scss';

export class LangSwitcher extends Component {
  static propTypes = {
    locale: PropTypes.string.isRequired,
    locales: PropTypes.oneOfType([PropTypes.objectOf(PropTypes.string), PropTypes.arrayOf(PropTypes.string)]),
    domain: PropTypes.string.isRequired,
    isWithinHeader: PropTypes.bool,
    isWhite: PropTypes.bool,
  };

  static defaultProps = {
    locales: Object.keys(LANG_NAMES),
    isWithinHeader: false,
    isWhite: false,
  };

  state = {
    dropdownOpen: false,
  };

  isMultiDomains = !Array.isArray(this.props.locales);

  toggle = () => {
    this.setState(prevState => ({
      dropdownOpen: !prevState.dropdownOpen,
    }));
  };

  changeLocale = (e) => {
    const { locale, locales, domain } = this.props;
    const newLocale = e.target.value;

    if (newLocale !== locale) {
      const url = this.isMultiDomains ?
        getLocationOrigin().replace(new RegExp(`(.*)${domain.toLowerCase()}`), `$1${locales[newLocale] || 'com'}`) :
        `${getLocationOrigin()}${window.location.pathname.replace(locale, newLocale)}`;

      changeLocation(url);
    }
  };

  render() {
    const { dropdownOpen } = this.state;
    const {
      locale,
      locales,
      isWithinHeader,
      isWhite,
    } = this.props;
    const localesList = this.isMultiDomains ? Object.keys(locales) : locales;

    return (
      <Dropdown
        isOpen={dropdownOpen}
        toggle={this.toggle}
        className={classNames('lang-switcher', { 'lang-switcher-within-header': isWithinHeader, 'lang-switcher-white': isWhite })}
      >
        <DropdownToggle
          nav
          className={classNames('lang-switcher-button p-0', { 'ml-1_5 text-gray-dark-secondary': !isWithinHeader })}
        >
          {!isWithinHeader ? `${LANG_NAMES[locale]}` : LANG_KEYS[locale].toUpperCase()}
          <IconArrowDown className={classNames('lang-switcher-arrow', { 'align-middle': isWithinHeader, 'position-relative': !isWithinHeader })} />
        </DropdownToggle>

        <DropdownMenu className="language-dropdown">
          {
            localesList.filter(loc => !!LANG_NAMES[loc]).map((loc) => {
              const isActive = loc === locale;

              return (
                <DropdownItem
                  key={loc}
                  value={loc}
                  active={isActive}
                  className={classNames('px-2', {
                    'font-weight-bold d-flex justify-content-between align-items-center category-selected': isActive,
                  })}
                  onClick={this.changeLocale}
                >
                  {LANG_NAMES[loc]}
                  {isActive && <IconSelected />}
                </DropdownItem>
              );
            })
          }
        </DropdownMenu>
      </Dropdown>
    );
  }
}
