import GA from './ga';
import * as generalHelpers from './general';

jest.spyOn(generalHelpers, 'canUseDOM');

const googleAnalyticsObject = 'testGAObj';
describe('', () => {
  window.gtag = jest.fn();
  window.fbq = jest.fn();
  window.GoogleAnalyticsObject = googleAnalyticsObject;
  window.testGAObj = jest.fn();
  window[googleAnalyticsObject] = jest.fn();

  beforeEach(() => {
    window.gtag.mockClear();
    window.fbq.mockClear();
    window[googleAnalyticsObject].mockClear();
  });

  describe('when canUseDOM', () => {
    beforeAll(() => {
      generalHelpers.canUseDOM.mockReturnValue(true);
    });

    describe('setUser', () => {
      test('should set user', () => {
        GA.setUser('test');
        expect(window.gtag).toHaveBeenCalledWith('set', {
          user_id: 'test',
        });
      });
    });

    describe('trackPage', () => {
      test('should track page', () => {
        GA.trackPage('test');
        expect(window[googleAnalyticsObject]).toHaveBeenCalledWith('set', 'page', 'test');
        expect(window[googleAnalyticsObject]).toHaveBeenCalledWith('send', 'pageview');
      });
    });

    describe('#event', () => {
      test('should event', () => {
        GA.event({});
        expect(window.gtag).toHaveBeenCalledWith('event', '', {
          event_category: '',
          event_label: '',
        });
        expect(window.fbq).toHaveBeenCalledWith('trackCustom', '_');
      });

      test('should event category/action', () => {
        GA.event({
          category: 'category',
          action: 'action',
        });
        expect(window.gtag).toHaveBeenCalledWith('event', 'action', {
          event_category: 'category',
          event_label: '',
        });
        expect(window.fbq).toHaveBeenCalledWith('trackCustom', 'category_action');
      });

      test('should event with label', () => {
        GA.event({
          category: 'cate gory',
          action: 'action',
          label: 'label',
        });
        expect(window.gtag).toHaveBeenCalledWith('event', 'action', {
          event_category: 'cate gory',
          event_label: 'label',
        });
        expect(window.fbq).toHaveBeenCalledWith('trackCustom', 'cate_gory_action');
      });
    });

    describe('when PROJECT_ENV is production', () => {
      beforeAll(() => {
        process.env.PROJECT_ENV = 'production';
      });

      afterAll(() => {
        process.env.PROJECT_ENV = 'development';
      });

      test('should not log', () => {
        console.log = jest.fn();

        GA.event({
          category: 'category',
          action: 'action',
        });
        GA.setUser('test');
        GA.trackPage('test');

        expect(console.log).not.toHaveBeenCalled();
      });
    });
  });

  describe('when NOT canUseDOM', () => {
    beforeAll(() => {
      generalHelpers.canUseDOM.mockReturnValue(false);
    });

    describe('setUser', () => {
      test('should not set user', () => {
        GA.setUser('test');
        expect(window.gtag).not.toHaveBeenCalled();
      });
    });

    describe('trackPage', () => {
      test('should not track page', () => {
        GA.trackPage('test');
        expect(window[googleAnalyticsObject]).not.toHaveBeenCalled();
      });
    });

    describe('#event', () => {
      test('should not event', () => {
        GA.event({
          category: 'category',
          action: 'action',
        });
        expect(window.gtag).not.toHaveBeenCalled();
        expect(window.fbq).not.toHaveBeenCalled();
      });

      test('should not event with label', () => {
        GA.event({
          category: 'category',
          action: 'action',
          label: 'label',
        });
        expect(window.gtag).not.toHaveBeenCalled();
        expect(window.fbq).not.toHaveBeenCalled();
      });
    });
  });
});
